//DHT22 Sensor Testing Code

#include "DHT.h"        // Importing DHT sensor library
//--------------------------------------------------------------------------
#define DHTPIN1 2        // Digital pin 2 connected to the DHT sensor
#define DHTPIN2 3
#define DHTTYPE1 DHT22   // Defining the type of sensor for the library
#define DHTTYPE2 DHT22

DHT dht1(DHTPIN1, DHTTYPE1);
DHT dht2(DHTPIN2, DHTTYPE2);

float LastKnownTemperature1;
float LastKnownHumidity1;
float LastKnownTemperature2;
float LastKnownHumidity2;
float LastKnownAverageTemperature;
float LastKnownAverageHumidity;
const char operationDelimiter = ' ';
const char settingDelimiter = ' ';
int mode = 0;
int alertToggle = 0;
float upperTempCap = 60;
float lowerTempCap = 0;
int dataRequest = 0;


void setup() {
  Serial.begin(9600);
  //Serial.println(F("Testing DHT22"));   //PrintF means the statis string is stored in program memory, not ram.
  dht1.begin();
  dht2.begin();
}


void loop() {

  // Wait 2 seconds before each read as the sensor update roughly every 2 seconds
  // Done in intervals 0.25 seconds while checking for user input in between. Makes the system more responsive.
  // This part also handles any data request, sends the last known temperature/humidity on request.
  
  for(int i=0;i<8;i++){
  delay(250);
  checkSerial_ConfigureMode();
  //processRequest1Sensor(LastKnownTemperature1, LastKnownHumidity1);;
  processRequest2Sensor(LastKnownTemperature1, LastKnownTemperature2, LastKnownHumidity1, LastKnownHumidity2);   
  }
  

  
  // Reading temperature/humidity can take up to 250 milliseconds.
  LastKnownTemperature1 = dht1.readTemperature();  // Reading the temperature as Celcius and storing in variable
  LastKnownHumidity1 = dht1.readHumidity();   // Reading the humitidy as a % and storing in variable humidity
  LastKnownTemperature2 = dht2.readTemperature();  // Reading the temperature as Celcius and storing in variable
  LastKnownHumidity2 = dht2.readHumidity();   // Reading the humitidy as a % and storing in variable humidity
  LastKnownAverageTemperature = (LastKnownTemperature1 + LastKnownTemperature2) / 2 ;
  LastKnownAverageHumidity = (LastKnownHumidity1 + LastKnownHumidity2) / 2 ;

  
  // Checking if any reads failed, exits early to loop again when failed
  if (isnan(LastKnownHumidity1) || isnan(LastKnownTemperature1) || isnan(LastKnownHumidity2) || isnan(LastKnownTemperature2) ) {
    Serial.println(F("Failed to read from DHT sensor"));
    return;
  }
  
  
  // Constant Data Mode
  if (mode == 1) {
    sendData2Sensor(LastKnownTemperature1, LastKnownTemperature2, LastKnownHumidity1, LastKnownHumidity2);
    //sendData1Sensor(LastKnownTemperature1, LastKnownHumidity1);
  }



  //Alert for low temp
  if ((alertToggle == 1) && (LastKnownAverageTemperature > upperTempCap)){
  //if ((alertToggle == 1) && (LastKnownTemperature1 > upperTempCap)){
    Serial.println("WARNING - HIGH TEMPERATURE DETECTED");
  }
  
  //Alert for high temp
  if ((alertToggle == 1) && (LastKnownAverageTempreture < lowerTempCap)){
  //if ((alertToggle ==1 ) && (LastKnownTemperature1 < lowerTempCap)){
    Serial.println("WARNING - LOW TEMPERATURE DETECTED");
  }

  



  //TRY THIS IF CHAR ARRAY DOESNT WORK
  /*
    //Sending sensor reading via serial using Strings, String objects should apparently be avoided in Arduino programs.
    String temperatureS = String(temperature);
    String humidityS = String(humidity);
    String stringToSend = "Temperature: " + temperatureS + " Humidity: " + humidityS;
    Serial.print(stringToSend);
    Serial.write(0x48);
  */

  //Used for displaying the sensor reading
  /*
    float heatIndex = dht.computeHeatIndex(temperature, humidity, false);   // Computing heat index in Celsius (isFahreheit = false)
    Serial.println(F("*********************************************************"));

    Serial.print(F("Temperature: "));   //Printing temperature
    Serial.print(temperature);
    Serial.println(F("°C"));

    Serial.print(F("Humidity: "));      //Printing humidity
    Serial.print(humidity);
    Serial.println(F("%"));

    Serial.print(F("Heat index: "));     //Printing heat index
    Serial.print(heatIndex);
    Serial.println(F("°C "));
    Serial.println(F("*********************************************************"));
    Serial.println();
  */

}




//This function will try to read any mode/warning configurations and then apply them
void checkSerial_ConfigureMode() {

  //checks if any data is in buffer
  if (Serial.available() > 0) {
    delay(25);
    //Serial.println("Starting To Read One Stream");
    char messageReceived[20] = "";
    byte index = 0;
    char * operation;
    char * setting;
    bool endOfLine = false;
    char character;

    //while loop for reading entire line until \n
    while (endOfLine == false) {
      if (Serial.available() > 0) {
        character = Serial.read();
      }
      messageReceived[index] = character;
      index++;
      if (character == '\n') {
        endOfLine = true;
        messageReceived[index] = '\0';
      }
    }
    //Serial.println("Ending To Read One Stream");
    //Serial.println(messageReceived);

    //parses and splits the input at the space
    operation = strtok(messageReceived, &operationDelimiter);
    setting = strtok(NULL, &settingDelimiter);
    
    //Serial.println("Break");
    //Serial.println(operation);
    //Serial.println(setting);


    //various recognized configurations/strings. if one of these configs are matched, the appropriate variables are set.
    if ((strcmp(operation, "mode") == 0) && (strcmp(setting, "1\n") == 0 || strcmp(setting, "0\n") == 0)) {
      if((strcmp(setting, "0\n") == 0)){
        Serial.println("MODE SET - DATA ON REQUEST");
      }
      else{
        Serial.println("MODE SET - CONTINUOUS DATA");
      }
      mode = atoi(setting);
    }

    else if ((strcmp(operation, "alert") == 0) && (strcmp(setting, "1\n") == 0 || strcmp(setting, "0\n") == 0)) {
      if((strcmp(setting, "0\n") == 0)){
        Serial.println("ALERTS OFF");
      }
      else{
        Serial.println("ALERTS ON");
      }
      alertToggle = atoi(setting);
    }
    
    else if ((strcmp(operation, "setU") == 0) && (atof(setting) >= 0 && atof(setting) <= 60)) {
      Serial.println("UPPER TEMP LIMIT SET");
      upperTempCap = atof(setting);
    }

    else if ((strcmp(operation, "setL") == 0) && (atof(setting) >= 0 && atof(setting) <= 60)) {
      Serial.println("LOWER TEMP LIMIT SET");
      lowerTempCap = atof(setting);
    }

    else if (((strcmp(operation, "r\n") == 0)) && mode == 0) {
      dataRequest = 1;
    }
    
    else{
      Serial.println("INVALID INPUT");
    }
    
  }
}




void processRequest1Sensor(float LastKnownTemperature1, float LastKnownHumidity1){  
  if ((mode == 0) && (dataRequest == 1)) {
    sendData1Sensor(LastKnownTemperature1, LastKnownHumidity1);
    dataRequest = 0;
  }
}



void processRequest2Sensor(float LastKnownTemperature1, float LastKnownTemperature2, float LastKnownHumidity1, float LastKnownHumidity2){ 
  if ((mode == 0) && (dataRequest == 1)) {
    sendData2Sensor(LastKnownTemperature1, LastKnownTemperature2, LastKnownHumidity1, LastKnownHumidity2);
    dataRequest = 0;
  }
}




void sendData2Sensor(float temperature1,float temperature2,float humidity1,float humidity2) {
  //Sending sensor reading via serial using Char arrays, this apparently stop heap fragmentation is overall better than using String objects.

  // Converting float data into char arrays.
  char temp[10];
  //size 10, dim 1
  //perform conversion
  dtostrf(temperature1, 1, 2, temp);
  //reading the temperature

  char temp2[10];
  //perform conversion
  dtostrf(humidity1, 1, 2, temp2);
  //reading the humidity from one sensor

  char temp3[10];
  //size 10, dim 1
  //perform conversion
  dtostrf(temperature2, 1, 2, temp3);
  //reading the temperature

  char temp4[10];
  //perform conversion
  dtostrf(humidity2, 1, 2, temp4);
  //reading the humidity from one sensor

  //Forming the full char array in the format "Temperature: 22.42 Humidity: 45%"
  char arrayToSend[80] = "";
  strcat(arrayToSend, "Temperature1: ");
  strcat(arrayToSend, temp);
  strcat(arrayToSend, " Humidity1: ");
  strcat(arrayToSend, temp2);
  strcat(arrayToSend, " Temperature2: ");
  strcat(arrayToSend, temp3);
  strcat(arrayToSend, " Humidity2: ");
  strcat(arrayToSend, temp4);
  strcat(arrayToSend, "\n");

  //Writing the Char array to the serial transfer line.
  Serial.write(arrayToSend);
}




void sendDataAverage(float temperatureMean, float humidityMean) {
  //Sending sensor reading via serial using Char arrays, this apparently stop heap fragmentation is overall better than using String objects.

  // Converting float data into char arrays.
  char temp[10];
  //size 10, dim 1
  //perform conversion
  dtostrf(temperatureMean, 1, 2, temp);
  //reading the temperature

  char temp2[10];
  //perform conversion
  dtostrf(humidityMean, 1, 2, temp2);
  //reading the humidity from one sensor

  //Forming the full char array in the format "Temperature: 22.42 Humidity: 45%"
  char arrayToSend[80] = "";
  strcat(arrayToSend, "Temperature Mean: ");
  strcat(arrayToSend, temp);
  strcat(arrayToSend, " Humidity Mean: ");
  strcat(arrayToSend, temp2);
  strcat(arrayToSend, "\n");

  //Writing the Char array to the serial transfer line.
  Serial.write(arrayToSend);
}




void sendData1Sensor(float temperature1, float humidity1) {
  //Sending sensor reading via serial using Char arrays, this apparently stop heap fragmentation is overall better than using String objects.

  // Converting float data into char arrays.
  char temp[10];
  //size 10, dim 1
  //perform conversion
  dtostrf(temperature1, 1, 2, temp);
  //reading the temperature

  char temp2[10];
  //perform conversion
  dtostrf(humidity1, 1, 2, temp2);
  //reading the humidity from one sensor

  //Forming the full char array in the format "Temperature: 22.42 Humidity: 45%"
  char arrayToSend[80] = "";
  strcat(arrayToSend, "Temperature: ");
  strcat(arrayToSend, temp);
  strcat(arrayToSend, " Humidity: ");
  strcat(arrayToSend, temp2);
  strcat(arrayToSend, "\n");

  //Writing the Char array to the serial transfer line.
  Serial.write(arrayToSend);
}
